package egg_energy

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	zhios_order_relate_utils "code.fnuoos.com/EggPlanet/egg_models.git/utils"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"time"
	"xorm.io/xorm"
)

// HomePageStartSignIn 主页发起签到
func HomePageStartSignIn(engine *xorm.Engine, req md.HomePageStartSignInReq) error {
	now := time.Now()
	zhios_order_relate_utils.FilePutContents("HomePageStartSignIn", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
		"body":       string(req.Body),
		"token":      req.Token,
		"user_phone": req.Phone,
		"user_uid":   req.UID,
		"time":       now.Format("2006-01-02 15:04:05"),
		"ip":         req.IP,
	}))

	//1、查找基础设置
	basicSettingDb := implement.NewEggEnergyBasicSettingDb(engine)
	basicSetting, err := basicSettingDb.EggEnergyBasicSettingGetOneByParams(map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return err
	}

	var directPushRewardSetting *md.DirectPushRewardSettingStruct
	err = json.Unmarshal([]byte(basicSetting.DirectPushReward), &directPushRewardSetting)
	if err != nil {
		return err
	}

	//TODO:: 查询当前用户观看视屏记录
	var videoRewardSystem *md.VideoRewardSystemStruct
	err = json.Unmarshal([]byte(basicSetting.VideoRewardSystem), &videoRewardSystem)
	if err != nil {
		return err
	}
	rewardTotalNum := zhios_order_relate_utils.StrToInt(videoRewardSystem.RewardTotalNum)
	recordsDb := implement.NewEggEnergyUserWatchRecordsDb(engine)
	records, err := recordsDb.EggEnergyUserWatchRecordsGetOneByParams(map[string]interface{}{
		"key":   "uid",
		"value": req.UID,
	})
	if err != nil {
		return err
	}
	var isCan bool
	if records != nil && records.ResidueWatchAdNum == rewardTotalNum && records.NextWatchAdDate.After(now) {
		//TODO::观看记录不为空, 剩余观看次数 = 一共X个奖励视屏, 下轮观看视屏时间在当前时间之后
		isCan = true
	}
	if !isCan {
		zhios_order_relate_utils.FilePutContents("HomePageStartSignInBad", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
			"body":       string(req.Body),
			"token":      req.Token,
			"header":     req.Header,
			"user_phone": req.Phone,
			"user_uid":   req.UID,
			"time":       now.Format("2006-01-02 15:04:05"),
			"ip":         req.IP,
		}))
		return errors.New("不满足签到条件")
	}

	signInDb := implement.NewEggSignInDb(engine)
	var oneRoundDuration = zhios_order_relate_utils.StrToInt(videoRewardSystem.EachRoundHour)
	startTime := now.Add(-time.Hour * time.Duration(oneRoundDuration)).Format("2006-01-02 15:04:05")
	get, eggSignIn, err := signInDb.EggSignINGetOneByTimeAndUid(startTime, "", req.UID, 0)
	if err != nil {
		return err
	}
	if get && zhios_order_relate_utils.String2Time(eggSignIn.EndTime).After(now) {
		zhios_order_relate_utils.FilePutContents("HomePageStartSignInERR", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
			"body":       string(req.Body),
			"token":      req.Token,
			"header":     req.Header,
			"user_phone": req.Phone,
			"user_uid":   req.UID,
			"time":       now.Format("2006-01-02 15:04:05"),
			"ip":         req.IP,
		}))
		return errors.New("非签到时间," + "请求账号：" + req.Phone + ",请求时间：" + now.Format("2006-01-02 15:04:05") + ",签到时间：" + eggSignIn.EndTime)
	}

	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		return err
	}
	defer ch.Release()

	err = ActivityCoinReadyExchangeEggEnergy(engine, req.UID, ch)
	if err != nil {
		return err
	}
	return nil
}
